/*
 *  jDTAUS Core Utilities
 *  Copyright (c) 2005 Christian Schulte (cs@jdtaus.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.monitor.util;

import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;

class SwingProgressMonitorBundle
{

  /** Singleton instance. */
  private static final SwingProgressMonitorBundle instance =
    new SwingProgressMonitorBundle();

  /** Creates a new {@code SwingProgressMonitorBundle} instance. */
  SwingProgressMonitorBundle()
  {
    this.getMessage ( "unknownEventType", Locale.getDefault() );
    this.getMessage ( "expectedEnd", Locale.getDefault() );
    this.getMessage ( "taskAlreadyStarted", Locale.getDefault() );
  }

  /**
   * Gets the {@code SwingProgressMonitorBundle} singleton.
   *
   * @return the {@code SwingProgressMonitorBundle} singleton.
   */
  static SwingProgressMonitorBundle getInstance()
  {
    return instance;
  }

  /**
   * Gets the text of the message with key {@code "unknownEventType"}.
   *
   * @return <pre>
   * A task event of unknown type {0} was ignored.</pre>
   */
  String getUnknownEventTypeText ( final Locale locale )
  {
    return this.getMessage ( "unknownEventType", locale );
  }

  /**
   * Gets the message with key {@code "unknownEventType"}.
   *
   * @return <pre>
   * A task event of unknown type {0} was ignored.</pre>
   */
  MessageFormat getUnknownEventTypeMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "unknownEventType", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "expectedEnd"}.
   *
   * @return <pre>
   * Expected end approximately at {0, date, full} {0, time, medium}.</pre>
   */
  String getExpectedEndText ( final Locale locale )
  {
    return this.getMessage ( "expectedEnd", locale );
  }

  /**
   * Gets the message with key {@code "expectedEnd"}.
   *
   * @return <pre>
   * Expected end approximately at {0, date, full} {0, time, medium}.</pre>
   */
  MessageFormat getExpectedEndMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "expectedEnd", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "taskAlreadyStarted"}.
   *
   * @return <pre>
   * A task with description {0} already has been started at {1,time,long}.</pre>
   */
  String getTaskAlreadyStartedText ( final Locale locale )
  {
    return this.getMessage ( "taskAlreadyStarted", locale );
  }

  /**
   * Gets the message with key {@code "taskAlreadyStarted"}.
   *
   * @return <pre>
   * A task with description {0} already has been started at {1,time,long}.</pre>
   */
  MessageFormat getTaskAlreadyStartedMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "taskAlreadyStarted", locale ), locale );
  }

  /**
   * Gets a message for a given key and locale.
   *
   * @param key the key of the message to return.
   * @param locale the locale of the message.
   *
   * @return message for {@code key} using {@code locale}.
   */
  private String getMessage( final String key, Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return ResourceBundle.getBundle( "org/jdtaus/core/monitor/util/SwingProgressMonitor",
                                     locale ).getString( key );

  }
}
