/*
 *  jDTAUS Core Utilities
 *  Copyright (c) 2005 Christian Schulte (cs@jdtaus.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.monitor.util;

import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;

class TaskDurationLoggerBundle
{

  /** Singleton instance. */
  private static final TaskDurationLoggerBundle instance =
    new TaskDurationLoggerBundle();

  /** Creates a new {@code TaskDurationLoggerBundle} instance. */
  TaskDurationLoggerBundle()
  {
    this.getMessage ( "durationInfo", Locale.getDefault() );
  }

  /**
   * Gets the {@code TaskDurationLoggerBundle} singleton.
   *
   * @return the {@code TaskDurationLoggerBundle} singleton.
   */
  static TaskDurationLoggerBundle getInstance()
  {
    return instance;
  }

  /**
   * Gets the text of the message with key {@code "durationInfo"}.
   *
   * @return <pre>
   * Taskinformation:
	Description: {0}
	Start: {1,date,long} at {1,time,long}
	End: {2,date,long} at {2,time,long}
	Duration: {3}ms</pre>
   */
  String getDurationInfoText ( final Locale locale )
  {
    return this.getMessage ( "durationInfo", locale );
  }

  /**
   * Gets the message with key {@code "durationInfo"}.
   *
   * @return <pre>
   * Taskinformation:
	Description: {0}
	Start: {1,date,long} at {1,time,long}
	End: {2,date,long} at {2,time,long}
	Duration: {3}ms</pre>
   */
  MessageFormat getDurationInfoMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "durationInfo", locale ), locale );
  }

  /**
   * Gets a message for a given key and locale.
   *
   * @param key the key of the message to return.
   * @param locale the locale of the message.
   *
   * @return message for {@code key} using {@code locale}.
   */
  private String getMessage( final String key, Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return ResourceBundle.getBundle( "org/jdtaus/core/monitor/util/TaskDurationLogger",
                                     locale ).getString( key );

  }
}
