/*
 *  jDTAUS Core Utilities
 *  Copyright (c) 2005 Christian Schulte (cs@jdtaus.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.text.util;

import java.util.Locale;
import java.util.ResourceBundle;
import java.text.MessageFormat;

class SwingMessagePaneBundle
{

  /** Singleton instance. */
  private static final SwingMessagePaneBundle instance =
    new SwingMessagePaneBundle();

  /** Creates a new {@code SwingMessagePaneBundle} instance. */
  SwingMessagePaneBundle()
  {
    this.getMessage ( "unknownMessageType", Locale.getDefault() );
    this.getMessage ( "warning", Locale.getDefault() );
    this.getMessage ( "information", Locale.getDefault() );
    this.getMessage ( "notification", Locale.getDefault() );
    this.getMessage ( "error", Locale.getDefault() );
  }

  /**
   * Gets the {@code SwingMessagePaneBundle} singleton.
   *
   * @return the {@code SwingMessagePaneBundle} singleton.
   */
  static SwingMessagePaneBundle getInstance()
  {
    return instance;
  }

  /**
   * Gets the text of the message with key {@code "unknownMessageType"}.
   *
   * @return <pre>
   * A message event of unknown type {0} was ignored.</pre>
   */
  String getUnknownMessageTypeText ( final Locale locale )
  {
    return this.getMessage ( "unknownMessageType", locale );
  }

  /**
   * Gets the message with key {@code "unknownMessageType"}.
   *
   * @return <pre>
   * A message event of unknown type {0} was ignored.</pre>
   */
  MessageFormat getUnknownMessageTypeMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "unknownMessageType", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "warning"}.
   *
   * @return <pre>
   * Warning</pre>
   */
  String getWarningText ( final Locale locale )
  {
    return this.getMessage ( "warning", locale );
  }

  /**
   * Gets the message with key {@code "warning"}.
   *
   * @return <pre>
   * Warning</pre>
   */
  MessageFormat getWarningMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "warning", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "information"}.
   *
   * @return <pre>
   * Information</pre>
   */
  String getInformationText ( final Locale locale )
  {
    return this.getMessage ( "information", locale );
  }

  /**
   * Gets the message with key {@code "information"}.
   *
   * @return <pre>
   * Information</pre>
   */
  MessageFormat getInformationMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "information", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "notification"}.
   *
   * @return <pre>
   * Notification</pre>
   */
  String getNotificationText ( final Locale locale )
  {
    return this.getMessage ( "notification", locale );
  }

  /**
   * Gets the message with key {@code "notification"}.
   *
   * @return <pre>
   * Notification</pre>
   */
  MessageFormat getNotificationMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "notification", locale ), locale );
  }
  /**
   * Gets the text of the message with key {@code "error"}.
   *
   * @return <pre>
   * Error</pre>
   */
  String getErrorText ( final Locale locale )
  {
    return this.getMessage ( "error", locale );
  }

  /**
   * Gets the message with key {@code "error"}.
   *
   * @return <pre>
   * Error</pre>
   */
  MessageFormat getErrorMessage ( Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return new MessageFormat ( this.getMessage ( "error", locale ), locale );
  }

  /**
   * Gets a message for a given key and locale.
   *
   * @param key the key of the message to return.
   * @param locale the locale of the message.
   *
   * @return message for {@code key} using {@code locale}.
   */
  private String getMessage( final String key, Locale locale )
  {
    if ( locale == null )
    {
      locale = Locale.getDefault();
    }

    return ResourceBundle.getBundle( "org/jdtaus/core/text/util/SwingMessagePane",
                                     locale ).getString( key );

  }
}
